@@ -5,10 +5,14 @@ from google.cloud.bigquery import SchemaField, Client, Row
 from google.cloud.bigquery.table import RowIterator
 
 from bigquery_frame.auth import get_bq_client
+from bigquery_frame.column import Column
 from bigquery_frame.has_bigquery_client import HasBigQueryClient
 from bigquery_frame.printing import print_results
 
 
+Column = Union[str, Column]
+
+
 def indent(str, nb) -> str:
     return " " * nb + str.replace("\n", "\n" + " " * nb)
 
@@ -26,6 +30,7 @@ def quote(str) -> str:
 
 
 def cols_to_str(cols, indentation: int = None) -> str:
+    cols = [str(col) for col in cols]
     if indentation is not None:
         return indent(",\n".join(cols), indentation)
     else:
@@ -44,6 +49,61 @@ def is_nullable(schema_field: SchemaField):
     return schema_field.mode == "NULLABLE"
 
 
+def schema_to_simple_string(schema: List[SchemaField]):
+    """Transforms a BigQuery DataFrame schema into a new schema where all structs have been flattened.
+    The field names are kept, with a '.' separator for struct fields.
+    If `explode` option is set, arrays are exploded with a '!' separator.
+
+    Example:
+
+    >>> bq = BigQueryBuilder(get_bq_client())
+    >>> df = bq.sql('SELECT 1 as id, STRUCT(1 as a, [STRUCT(2 as c, 3 as d)] as b, [4, 5] as e) as s')
+    >>> print(df.schema)
+    [SchemaField('id', 'INTEGER', 'NULLABLE', None, (), None), SchemaField('s', 'RECORD', 'NULLABLE', None, (SchemaField('a', 'INTEGER', 'NULLABLE', None, (), None), SchemaField('b', 'RECORD', 'REPEATED', None, (SchemaField('c', 'INTEGER', 'NULLABLE', None, (), None), SchemaField('d', 'INTEGER', 'NULLABLE', None, (), None)), None), SchemaField('e', 'INTEGER', 'REPEATED', None, (), None)), None)]
+    >>> schema_to_simple_string(df.schema)
+    'id:INTEGER,s:STRUCT<a:INTEGER,b:ARRAY<STRUCT<c:INTEGER,d:INTEGER>>,e:ARRAY<INTEGER>>'
+
+    :param schema:
+    :return:
+    """
+    def schema_field_to_simple_string(schema_field: SchemaField):
+        if is_struct(schema_field):
+            if is_repeated(schema_field):
+                return f"ARRAY<STRUCT<{schema_to_simple_string(schema_field.fields)}>>"
+            else:
+                return f"STRUCT<{schema_to_simple_string(schema_field.fields)}>"
+        else:
+            if is_repeated(schema_field):
+                return f"ARRAY<{schema_field.field_type}>"
+            else:
+                return schema_field.field_type
+
+    cols = [f"{field.name}:{schema_field_to_simple_string(field)}" for field in schema]
+    cols_to_string = ",".join(cols)
+    return f"{cols_to_string}"
+
+
+def schema_to_tree_string(schema: List[SchemaField]) -> str:
+    """Generates a string representing the schema in tree format"""
+
+    def str_gen_schema_field(schema_field: SchemaField, prefix: str) -> List[str]:
+        res = [f"{prefix}{schema_field.name}: {schema_field.field_type} ({schema_field.mode})"]
+        if is_struct(schema_field):
+            res += str_gen_schema(schema_field.fields, " |   " + prefix)
+        return res
+
+    def str_gen_schema(schema: List[SchemaField], prefix: str) -> List[str]:
+        return [
+            str
+            for schema_field in schema
+            for str in str_gen_schema_field(schema_field, prefix)
+        ]
+
+    res = ["root"] + str_gen_schema(schema, " |-- ")
+
+    return "\n".join(res) + "\n"
+
+
 def _dedup_key_value_list(l: List[Tuple[object, object]]):
     """Deduplicate a list of key, values by their keys.
     Unlike `list(set(l))`, this does preserve ordering.
@@ -189,14 +249,28 @@ class DataFrame:
         """
         self.bigquery._registerDataFrameAsTable(self, name)
 
-    def select(self, *columns: Union[List[str], str]) -> 'DataFrame':
+    def select(self, *columns: Union[List[Column], Column]) -> 'DataFrame':
         """Projects a set of expressions and returns a new :class:`DataFrame`."""
-        if len(columns) == 1 and isinstance(columns[0], list):
-            columns = columns[0]
-        col_str = ',\n'.join(columns)
+        if isinstance(columns[0], list):
+            if len(columns) == 1:
+                columns = columns[0]
+            else:
+                raise TypeError(f"Wrong argument type: {type(columns)}")
         query = strip_margin(
             f"""SELECT 
-            |{indent(col_str, 2)}
+            |{cols_to_str(columns, 2)}
+            |FROM {self._alias}""")
+        return self._apply_query(query)
+
+    def drop(self, *cols: str) -> 'DataFrame':
+        """Returns a new :class:`DataFrame` that drops the specified column.
+        This is a no-op if schema doesn't contain the given column name(s).
+        """
+        schema_cols = set(self.columns)
+        cols = [col for col in cols if col in schema_cols]
+        query = strip_margin(
+            f"""SELECT 
+            |  * EXCEPT ({cols_to_str(cols)})
             |FROM {self._alias}""")
         return self._apply_query(query)
 
@@ -241,12 +315,12 @@ class DataFrame:
         |  4   |  5   |  6   |
         +------+------+------+
         >>> df1.unionByName(df2).show()
-        +------+------+------+------+
-        | col0 | col1 | col2 | col3 |
-        +------+------+------+------+
-        |  1   |  2   |  3   | null |
-        | null |  4   |  5   |  6   |
-        +------+------+------+------+
+        +------+------+------+
+        | col0 | col1 | col2 |
+        +------+------+------+
+        |  1   |  2   |  3   |
+        |  6   |  4   |  5   |
+        +------+------+------+
 
         When the parameter `allowMissingColumns` is ``True``, the set of column names
         in this and other :class:`DataFrame` can differ; missing columns will be filled with null.
@@ -259,8 +333,8 @@ class DataFrame:
         +------+------+------+------+
         | col0 | col1 | col2 | col3 |
         +------+------+------+------+
-        |  1   |  2   |  3   |      |
-        |      |  4   |  5   |  6   |
+        |  1   |  2   |  3   | null |
+        | null |  4   |  5   |  6   |
         +------+------+------+------+
 
         :param other: Another DataFrame
@@ -306,6 +380,16 @@ class DataFrame:
         query = f"""SELECT * FROM {self._alias} LIMIT {num}"""
         return self._apply_query(query)
 
+    def distinct(self) -> 'DataFrame':
+        """Returns a new :class:`DataFrame` containing the distinct rows in this :class:`DataFrame`.
+
+        Limitations compared to Spark
+        -----------------------------
+        In BigQuery, the DISTINCT statement does not work on complex types like STRUCT and ARRAY.
+        """
+        query = f"""SELECT DISTINCT * FROM {self._alias}"""
+        return self._apply_query(query)
+
     def sort(self, *cols: str):
         """Returns a new :class:`DataFrame` sorted by the specified column(s).
 
@@ -400,25 +484,9 @@ class DataFrame:
         res = self.limit(n).collect_iterator()
         print_results(res, format_args)
 
-    def treeString(self) -> str:
+    def treeString(self):
         """Generates a string representing the schema in tree format"""
-
-        def str_gen_schema_field(schema_field: SchemaField, prefix: str) -> List[str]:
-            res = [f"{prefix}{schema_field.name}: {schema_field.field_type} ({schema_field.mode})"]
-            if is_struct(schema_field):
-                res += str_gen_schema(schema_field.fields, " |   " + prefix)
-            return res
-
-        def str_gen_schema(schema: List[SchemaField], prefix: str) -> List[str]:
-            return [
-                str
-                for schema_field in schema
-                for str in str_gen_schema_field(schema_field, prefix)
-            ]
-
-        res = ["root"] + str_gen_schema(self.schema, " |-- ")
-
-        return "\n".join(res) + "\n"
+        return schema_to_tree_string(self.schema)
 
     def printSchema(self) -> None:
         """Prints out the schema in tree format.
